﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Http;
using HotelService.Data;
using HotelService.Handler;
using System.Diagnostics;
using HotelService.Formatter;
//using System.Net.Http.Formatting;
using System.Net.Http.Headers;
using System.Net.Http.Formatting;
using System.Net.Http.Handlers;
using System.IO;

namespace Client
{

    class LoggingMessageHandler : MessageProcessingHandler
    {
        public LoggingMessageHandler()
            : base()
        {
            this.InnerHandler = new HttpClientHandler();
        }

        public LoggingMessageHandler(MessageProcessingHandler h) : base(h) { }

        protected override HttpRequestMessage ProcessRequest(HttpRequestMessage request, System.Threading.CancellationToken cancellationToken)
        {
            Debug.WriteLine("Request: " + request.RequestUri.ToString());
            return request;

        }

        protected override HttpResponseMessage ProcessResponse(HttpResponseMessage response, System.Threading.CancellationToken cancellationToken)
        {
            Debug.WriteLine("Response: " + response.StatusCode);
            return response;
        }
    }

    class Program
    {

        static async void DownloadDemo()
        {
 

            var progressMessageHandler = new ProgressMessageHandler()
            {
                InnerHandler = new HttpClientHandler()
            };

            progressMessageHandler.HttpReceiveProgress += progressMessageHandler_HttpReceiveProgress;
            
            HttpClient client = new HttpClient(progressMessageHandler);

            //var stream = await client.GetStreamAsync("http://localhost:1307/api/Bilder");

            var response = await client.GetAsync("http://localhost:1307/api/Bilder");
            var stream = await response.Content.ReadAsStreamAsync();

            Console.WriteLine("Habe Stream!");
            

            using (FileStream fs = new FileStream(@"c:\temp\download.dat", FileMode.Create))
            {
                int counter = 0;
                byte[] buffer = new byte[200*1024];

                
                while (stream.Read(buffer, 0, buffer.Length) > 0)
                {
                    counter++;
                    Console.WriteLine(counter * 200 * 1024 + " Bytes gelesen");
                }
            }
            Console.WriteLine("Fertig gelesen");

        }

        static void progressMessageHandler_HttpReceiveProgress(object sender, HttpProgressEventArgs e)
        {
            if (progress == e.ProgressPercentage) return;
            Console.WriteLine(e.ProgressPercentage + "% heruntergeladen");
            progress = e.ProgressPercentage;
        }

        static async void UploadDemo()
        {

            var fileName = @"c:\temp\4upload.dat";
            var fileStream = new FileStream(fileName, FileMode.Open);

            var progressMessageHandler = new ProgressMessageHandler() {
                InnerHandler = new HttpClientHandler()
            };

            

            progressMessageHandler.HttpSendProgress += progressMessageHandler_HttpSendProgress;


            
            var content = new MultipartFormDataContent();
            content.Add(new StreamContent(fileStream), "file", "something.dat");

            var message = new HttpRequestMessage();
            message.Method = HttpMethod.Post;
            message.RequestUri = new Uri("http://localhost:1307/api/Bilder");
            message.Content = content;

            HttpClient client = new HttpClient(progressMessageHandler);
            var response = await client.SendAsync(message);
            var text = await response.Content.ReadAsStringAsync();

            Console.WriteLine(text);
        }

        static async void UploadWithStreamingDemo()
        {
            var url = "http://localhost:1307/api/Bilder";
            //var url = "http://IWI-NB-07:8080/api/Bilder";

            var fileName = @"c:\temp\4upload.dat";
            var fileStream = new FileStream(fileName, FileMode.Open);

            HttpClient client = new HttpClient();
            var request = new HttpRequestMessage();
            request.Method = HttpMethod.Put;
            request.RequestUri = new Uri(url);
            request.Content = new StreamContent(fileStream);

            var response = await client.SendAsync(request);
            var text = await response.Content.ReadAsStringAsync();

            Console.WriteLine(text);
        }


        static int progress = -1;
        static void progressMessageHandler_HttpSendProgress(object sender, HttpProgressEventArgs e)
        {
            if (progress == e.ProgressPercentage) return;
            Console.WriteLine(e.ProgressPercentage + "% hochgeladen");
            progress = e.ProgressPercentage;
        }

static async void SimpleGet()
{

    var client = new HttpClient();

    client.DefaultRequestHeaders.AcceptLanguage.Add(new StringWithQualityHeaderValue("de-DE"));
    client.DefaultRequestHeaders.AcceptLanguage.Add(new StringWithQualityHeaderValue("de-AT"));

    var url = "http://localhost:1307/api/Hotels";
    var response = await client.GetAsync(url);
    var hotels = await response.Content.ReadAsAsync<IEnumerable<Hotel>>();



    foreach (var hotel in hotels)
    {
        Console.WriteLine(hotel.Bezeichnung);
    }

            /*
            client.GetAsync(url).ContinueWith(getTask =>
            {
                getTask.Result.Content.ReadAsAsync<IEnumerable<Hotel>>().ContinueWith(readTask =>
                {
                    var hotels = readTask.Result;
                    foreach (var hotel in hotels)
                    {
                        Console.WriteLine(hotel.Bezeichnung);
                    }

                });
            });
            */
        }

        static void Main(string[] args)
        {

            Console.WriteLine("Drücken Sie eine Taste um zu starten ...");
            Console.ReadLine();

            // SimpleGet();
            // SendHotel();
            // SendBuchung();

            // GetWithHandlerAndFormatterDemo();
            // UploadDemo();
            DownloadDemo();
            // UploadWithStreamingDemo();
            

            Console.WriteLine("Aktion wird im Hintergrund ausgeführt. Drücken Sie eine Taste um die Anwendung zu schließen ...\n");
            Console.ReadLine();

        }

        private async static void SendBuchung()
        {
            var url = "http://localhost:1307/api/Buchungen";

            var buchung = new HotelBuchung
            {
                HotelId = 1,
                Vorname = "Max",
                Nachname = "Muster"
            };

            var client = new HttpClient();
            

            // RC:
            // var request = new HttpRequestMessage<HotelBuchung>();
            var request = new HttpRequestMessage();
            request.Content = new ObjectContent<HotelBuchung>(buchung, new JsonMediaTypeFormatter());
            request.Content.Headers.ContentType = new MediaTypeHeaderValue("text/json");


            request.Method = HttpMethod.Post;
            request.RequestUri = new Uri(url);
            

            request.Headers.AcceptLanguage.Add(new StringWithQualityHeaderValue("de-DE"));
            request.Headers.AcceptLanguage.Add(new StringWithQualityHeaderValue("de-AT"));

            var response = await client.SendAsync(request);

            Console.WriteLine("Status: " + response.StatusCode);
            Console.WriteLine("Location: " + response.Headers.Location);

            // Anfrage senden
            //client.SendAsync(request).ContinueWith(task =>
            //{
            //    Console.WriteLine("Status: " + task.Result.StatusCode);
            //    Console.WriteLine("Location: " + task.Result.Headers.Location);
            //     Payload lesen: task.Result.Content.ReadAsAsync<...>
            //});

        }



        private async static void SendHotel()
        {
            var url = "http://localhost:1307/api/Hotels";

            var hotel = new Hotel
            {
                HotelId = 100,
                Bezeichnung = "Hotel zur Bruchbude",
                Sterne = 1
            };

            // Liste mit Formatter bereitstellen
            var formatters = new List<MediaTypeFormatter>();
            var formatter = new FlatFileFormatter();
            formatters.Add(formatter);


            var client = new HttpClient();
            
            // RC
            // var request = new HttpRequestMessage<List<Hotel>>(new List<Hotel> { hotel }, new MediaTypeHeaderValue("text/csv"), formatters);
            var request = new HttpRequestMessage();
            request.Content = new ObjectContent<List<Hotel>>(new List<Hotel> { hotel }, formatter);
            request.Content.Headers.ContentType = new MediaTypeHeaderValue("text/csv");

            //request.Method = HttpMethod.Post;
            request.Method = new HttpMethod("X-ECHO");
            request.RequestUri = new Uri(url);

            

            request.Headers.Accept.Add(new MediaTypeWithQualityHeaderValue("text/csv"));
            request.Headers.AcceptLanguage.Add(new StringWithQualityHeaderValue("de-DE"));
            request.Headers.AcceptLanguage.Add(new StringWithQualityHeaderValue("de-AT"));


            var response = await client.SendAsync(request);

            Console.WriteLine("Status: " + response.StatusCode);
            Console.WriteLine("Location: " + response.Headers.Location);

            var hotels = await response.Content.ReadAsAsync<IEnumerable<Hotel>>(formatters);

            foreach (var h in hotels)
            {
                Console.WriteLine(h.Bezeichnung);
            }


            // Anfrage senden
            //client.SendAsync(request).ContinueWith(task =>
            //{
            //    Console.WriteLine("Status: " + task.Result.StatusCode);
            //    Console.WriteLine("Location: " + task.Result.Headers.Location);

            //    //Console.WriteLine(task.Result.Content.ReadAsStringAsync().Result);

            //    task.Result.Content.ReadAsAsync<IEnumerable<Hotel>>(formatters).ContinueWith(task2 =>
            //    {
            //        var hotels = task2.Result;

            //        foreach (var h in hotels)
            //        {
            //            Console.WriteLine(h.Bezeichnung);
            //        }
            //    });
            //});

        }

        private async static void GetWithHandlerAndFormatterDemo()
        {
            // Handler instanziieren und verketten
            var handler = new LoggingMessageHandler
            {
                InnerHandler = new HttpClientHandler()
            };

            

            // Client erzeugen
            //var client = new HttpClient(handler);

            // Alternative mit Factory
            var client = HttpClientFactory.Create(new LoggingMessageHandler());


            // Liste mit Formatter bereitstellen
            var formatters = new List<MediaTypeFormatter>();
            formatters.Add(new FlatFileFormatter());

            // Accept-Header auf text/csv setzen
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("text/csv"));

            var url = "http://localhost:1307/api/Hotels";

            // Daten anfordern
            var response = await client.GetAsync(url);
            var hotels = await response.Content.ReadAsAsync<IEnumerable<Hotel>>(formatters);

            foreach (var hotel in hotels)
            {
                Console.WriteLine(hotel.Bezeichnung);
            }

            //client.GetAsync(url).ContinueWith(getTask =>
            //{
            //    // Daten unter Verwendung der Formatter lesen
            //    getTask.Result.Content.ReadAsAsync<IEnumerable<Hotel>>(formatters).ContinueWith(readTask =>
            //    {
            //        var hotels = readTask.Result;
            //        foreach (var hotel in hotels)
            //        {
            //            Console.WriteLine(hotel.Bezeichnung);
            //        }

            //    });
            //});
        }
    }
}
