﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Web.Http;
using HotelService.Data;
using System.Net;
using System.Net.Http.Headers;
using System.IO;
using System.Diagnostics;

namespace HotelService.Controllers
{

    public class BuchungenController : ApiController
    {

        public HttpResponseMessage Post(HotelBuchung buchung)
        {

            var rep = new HotelBuchungRepository();
            rep.Create(buchung);

            var response = new HttpResponseMessage(HttpStatusCode.Created);
            string uri = Url.Route(null, new { id = buchung.HotelBuchungId });
            response.Headers.Location = new Uri(Request.RequestUri, uri);

            return response;

        }

        // RC: HttpResponseMessage nicht mehr generisch 
        // public HttpResponseMessage<List<HotelBuchung>> GetByHotel(int hotelId)
        public HttpResponseMessage GetByHotel(int hotelId)
        {
            var rep = new HotelBuchungRepository();
            var buchungen = rep.FindByHotel(hotelId);
            
            // RC: ?? How to use ObjectContent new - expects a MediaTypeFormatter in constructor ...
            // var response = new HttpResponseMessage();
            // response.Content = new ObjectContent<List<HotelBuchung>>(buchungen);

            
            
            

            var response = Request.CreateResponse<List<HotelBuchung>>(HttpStatusCode.OK, buchungen);

            response.Headers.CacheControl = new CacheControlHeaderValue();
            response.Headers.CacheControl.NoCache = true;
            response.StatusCode = HttpStatusCode.OK;

            return response;

        }

        private string Quote(string str)
        {
            return "\"" + str + "\"";
        }

        // BETA1: In der finalen Version soll die HttpRequestMessage in die Methode injiziert werden.
        // In der aktuellen BETA muss man stattdessen auf this.Request zurückgreifen.
        
        // RC
        // public HttpResponseMessage<HotelBuchung> Get(int id/*, HttpRequestMessage request*/)
        public HttpResponseMessage Get(int id, HttpRequestMessage request)
        {
            // RC: Injizieren von HttpRequestMessage funktioniert nun
            // var request = this.Request; // HACK für BETA1

            var rep = new HotelBuchungRepository();
            var buchung = rep.FindById(id);

            if (buchung == null)
            {
                // RC
                //return new HttpResponseMessage<HotelBuchung>(HttpStatusCode.NotFound);

                return this.Request.CreateResponse(HttpStatusCode.NotFound);
            }

            var etag = Quote(buchung.Version.ToString());
            var ifNoneMatchHeader = request.Headers.IfNoneMatch.FirstOrDefault();
            if (ifNoneMatchHeader != null && ifNoneMatchHeader.Tag == etag)
            {
                // RC
                // return new HttpResponseMessage<HotelBuchung>(HttpStatusCode.NotModified);
                return this.Request.CreateResponse(HttpStatusCode.NotModified);
            }

            // RC
            // var response = new HttpResponseMessage<HotelBuchung>(buchung);

            var response = this.Request.CreateResponse<HotelBuchung>(HttpStatusCode.OK, buchung);

            response.StatusCode = HttpStatusCode.OK;
            response.Headers.CacheControl = new CacheControlHeaderValue();
            response.Headers.CacheControl.NoCache = true;
            response.Headers.ETag = new EntityTagHeaderValue(etag);
            return response;

        }


        // BETA1: Injizieren von HttpRequestMessage funktioniert noch nicht !!!
        // RC: Injizieren von HttpRequestMessage funktioniert nun!
        // IIS8: Nicht vergessen: Verben in IIS-Config freigeben
        public void Put([FromUri]int id, HttpRequestMessage request)
        {
            // RC: funktioniert nun!
            // var request = this.Request; // HACK für BETA1

            var stream = request.Content.ReadAsStreamAsync().Result;

            

            using (var r = new StreamReader(stream)) {
                Debug.WriteLine(r.ReadToEnd());
            }
        }
    }
}
