﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Runtime.Serialization;
using System.Web;
using System.Web.Http;
using System.Web.Http.Hosting;
using System.Web.Http.WebHost;
using System.Web.Mvc;
using System.Web.Optimization;
using System.Web.Routing;
using HotelService.Formatter;
using HotelService.Handler;
using HotelService.Resolver;
using WebAPI_RC1.Controllers;

namespace WebAPI_RC1
{
    // Note: For instructions on enabling IIS6 or IIS7 classic mode, 
    // visit http://go.microsoft.com/?LinkId=9394801

    public class CustomWebHostBufferPolicySelector : WebHostBufferPolicySelector
    {
        public override bool UseBufferedInputStream(object hostContext)
        {
            HttpContextBase contextBase = hostContext as HttpContextBase;

            if (contextBase != null)
            {
                RouteData routeData = contextBase.Request.RequestContext.RouteData;

                if (routeData.Values["controller"].ToString().ToLower() == "bilder")
                {
                    return false;
                }
            }

            return true;
        }

        public override bool UseBufferedOutputStream(System.Net.Http.HttpResponseMessage response)
        {
            return base.UseBufferedOutputStream(response);
        }
    } 



    public class WebApiApplication : System.Web.HttpApplication
    {
        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();

            FilterConfig.RegisterGlobalFilters(GlobalFilters.Filters);
            RouteConfig.RegisterRoutes(RouteTable.Routes);
            BundleConfig.RegisterBundles(BundleTable.Bundles);

            GlobalConfiguration.Configuration.IncludeErrorDetailPolicy = IncludeErrorDetailPolicy.LocalOnly;
            

            
            
            GlobalConfiguration.Configuration.Formatters.Add(new FlatFileFormatter());



            //GlobalConfiguration.Configuration.IncludeErrorDetailPolicy = IncludeErrorDetailPolicy.Always;

            //GlobalConfiguration.Configuration.MessageHandlers.Add(new FlatFileExtensionHandler());
            GlobalConfiguration.Configuration.MessageHandlers.Add(new MethodOverrideHandler());
            GlobalConfiguration.Configuration.MessageHandlers.Add(new LoggingHandler());

            // GlobalConfiguration.Configuration.DependencyResolver = new CustomResolver();

            // GlobalConfiguration.Configuration.ServiceResolver.SetResolver(new CustomResolver());

            //DependencyResolver.SetResolver(new CustomResolver());


            GlobalConfiguration.Configuration.Services.Replace(typeof(IHostBufferPolicySelector), new CustomWebHostBufferPolicySelector());


            var jf = GlobalConfiguration.Configuration.Formatters.JsonFormatter;
            // jf.UseDataContractJsonSerializer = true;
            jf.SerializerSettings.DateTimeZoneHandling = Newtonsoft.Json.DateTimeZoneHandling.RoundtripKind;
            jf.SerializerSettings.DateFormatHandling = Newtonsoft.Json.DateFormatHandling.IsoDateFormat;
            // jf.SerializerSettings.DateFormatHandling = Newtonsoft.Json.DateFormatHandling.MicrosoftDateFormat;
            jf.SerializerSettings.Formatting = Newtonsoft.Json.Formatting.Indented;
            
            //jf.MaxDepth = 255;
            //jf.SerializerSettings.MissingMemberHandling = Newtonsoft.Json.MissingMemberHandling.Error;

            // Geht nicht. Problem mit JSON.net-Version ??
            //jf.SerializerSettings.ContractResolver = new CamelCasePropertyNameContractResolver();

            jf.SerializerSettings.PreserveReferencesHandling = Newtonsoft.Json.PreserveReferencesHandling.All;
            

            var xf = GlobalConfiguration.Configuration.Formatters.XmlFormatter;

            xf.UseXmlSerializer = true;
            xf.Indent = true;

            var dcsAbteilung = new DataContractSerializer(typeof(Abteilung), null, int.MaxValue,
                false, /* preserveObjectReferences: */ true, null);
            xf.SetSerializer<Abteilung>(dcsAbteilung);
            
            var dcsMitarbeiter = new DataContractSerializer(typeof(Mitarbeiter), null, int.MaxValue,
                false, /* preserveObjectReferences: */ true, null);
            xf.SetSerializer<Mitarbeiter>(dcsMitarbeiter);


        }
    }
}