﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.Mvc;
using System.Web.Optimization;
using MvcApplication7.Models;

namespace MvcApplication7.Controllers
{
    public class HomeController : Controller
    {
        public ActionResult Index()
        {
            ViewBag.Message = "Modify this template to jump-start your ASP.NET MVC application.";

            return View();
        }

        

        public ActionResult CacheManifest()
        {
            bool enabled = IsAppCacheEnabled();

            if (!enabled)
            {
                Response.StatusCode = 410; //NotFound
                return new EmptyResult();
            }

            SetHeaders();

            var timeStamp = GetTimestamp();
            var cacheList = GetCacheList();

            var model = new CacheManifestModel();
            model.FilesToCache = cacheList;
            model.AssemblyTimestamp = timeStamp.ToString();

            return PartialView("CacheManifest", model);
        }

        private static bool IsAppCacheEnabled()
        {
            bool enableAppCache;
            var setting = System.Configuration.ConfigurationManager.AppSettings["EnableAppCache"];
            if (!string.IsNullOrEmpty(setting) && setting.ToLower().Trim() == "true")
            {
                enableAppCache = true;
            }
            else
            {
                enableAppCache = false;
            }
            return enableAppCache;
        }

        private List<string> GetCacheList()
        {
            var imagesToCache = Directory
                                    .GetFiles(Server.MapPath("~/Images"))
                                    .Select(f => "~/Images/" + Path.GetFileName(f));

            var bundlesToCache = BundleTable
                                    .Bundles
                                    .Select(b => b.Path).ToList();


            var cacheList = new List<string>();
            cacheList.AddRange(imagesToCache);
            cacheList.AddRange(bundlesToCache);
            return cacheList;
        }

        private string GetTimestamp()
        {
            var path = this.GetType().Assembly.CodeBase;
            var file = new FileInfo(path.Replace("file:///", ""));
            var timeStamp = file.LastWriteTime.ToString();
            return timeStamp;
        }

        private void SetHeaders()
        {
            Response.ContentType = "text/cache-manifest";
            Response.Cache.SetExpires(DateTime.UtcNow.AddDays(-1));
            Response.Cache.SetSlidingExpiration(false);
            Response.Cache.SetValidUntilExpires(false);
            Response.Cache.SetRevalidation(HttpCacheRevalidation.AllCaches);
            Response.Cache.SetCacheability(HttpCacheability.NoCache);
            //Response.Cache.SetNoStore(); 
        }


        public ActionResult About()
        {
            ViewBag.Message = "Your app description page.";

            return View();
        }

        public ActionResult Contact()
        {
            ViewBag.Message = "Your contact page.";

            return View();
        }
    }
}
