﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNet.SignalR.Client.Hubs;


namespace Client
{


    class Program
    {
        static void Main(string[] args)
        {

            Console.WriteLine("[Enter]: Start");
            Console.ReadLine();

            var connection = new HubConnection("http://localhost:15923");
            //var connection = new HubConnection("http://localhost:8081");

            
            
            
            
            IHubProxy proxy = connection.CreateHubProxy("MulticastFlugHub");

            proxy["name"] = "max";

            proxy.On("bookFlightCompleted", 
                (int flugId, string ticketId, string vorname, string nachname) => {

                    Console.WriteLine(
                        "bookFlightCompleted: flugId={0}, ticketId={1}, vorname={2}, nachname={3}", 
                        flugId, ticketId, vorname, nachname);

            });

            proxy.On("flightUpdated", (int flugId, string ticketId, int free) => {

                Console.WriteLine(
                    "flightUpdated: flugId={0}, ticketId={1}, free={2}",
                    flugId, ticketId, free);
            });

            proxy.On("status", (string status) => {
                Console.WriteLine("Status: " + status);
            });

            proxy["Address"] = "Tribstrühl 47, 1024 Krummeck";

            

            connection.Error += connection_Error;


            connection.Credentials = new NetworkCredential("max", "geheim");
            

            //connection.CookieContainer = new CookieContainer();
            //connection.CookieContainer.Add(new Cookie(
            //    name: "userName",
            //    value: "max",
            //    path: "/",
            //    domain: "localhost"));

            //connection.CookieContainer.Add(new Cookie(
            //    name: "password",
            //    value: "geheim",
            //    path: "/",
            //    domain: "localhost"));


            connection.Start().ContinueWith(t =>
            {
                if (t.Exception != null)
                {
                    t.Exception.Handle(e =>
                    {
                        Console.WriteLine(e.Message);
                        return true;
                    });
                }
                else
                {
                    Console.WriteLine("Verbindung aufgebaut!");
                }

            });


            
            

            string line;

            while ((line = Console.ReadLine()) != "exit")
            {
                if (string.IsNullOrWhiteSpace(line)) continue;
                
                var parts = line.Split(' ');
                var command = parts[0];

                int flugId = 0;
                switch (command)
                {
                    case "book":
                        flugId = Convert.ToInt32(parts[1]);
                        var vorname = parts[2];
                        var nachname = parts[3];

                        // proxy.Invoke("BookFlight", flugId, vorname, nachname);

                        proxy
                            .Invoke<string>("BookFlight", flugId, vorname, nachname)
                            .ContinueWith(t => {
                                Console.WriteLine("TicketId: " + t.Result);
                                
                            });
                            
                        // t.Exception vs. t.Result !!

                        break;

                    case "register":
                        flugId = Convert.ToInt32(parts[1]);
                        proxy.Invoke("RegisterForFlight", flugId);
                        break;

                    case "unregister":
                        flugId = Convert.ToInt32(parts[1]);
                        proxy.Invoke("UnRegisterForFlight", flugId);
                        break;

                    default:
                        Console.WriteLine("Mögliche Kommandos: book flugId vorname nachname | register flugId | unregister flugId");
                        break;
                }
            }
            connection.Stop();
        }

        static void connection_Error(Exception obj)
        {
            Console.WriteLine("Fehler: " + obj.Message);
        }
    }
}
