﻿using Microsoft.AspNet.SignalR.Hubs;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using Microsoft.AspNet.SignalR;

namespace SignalRDemo.Hubs
{
    public class MulticastFlugHub : Hub
    {
        // FlugId --> Anzahl freier Plätze
        private static Dictionary<int, int> freePerFlight = new Dictionary<int, int>();
        private static int nextTicket = 1;

        static MulticastFlugHub()
        {
            freePerFlight[4711] = 100;
            freePerFlight[4712] = 100;
            freePerFlight[4713] = 100;
        }

        // [Authorize] 
        public void RegisterForFlight(int flugId)
        {
            this.Groups.Add(this.Context.ConnectionId, "flug" + flugId);
            
        }

        public void UnRegisterForFlight(int flugId)
        {
            this.Groups.Remove(this.Context.ConnectionId, "flug" + flugId);
            
        }

        public string BookFlight(int flugId, string vorname, string nachname)
        {
            int free;
            string ticketId;

            string name = Clients.Caller.name;

            lock (freePerFlight)
            {
                if (!freePerFlight.ContainsKey(flugId)) throw new FlugException("Flug " + flugId + " existiert nicht!") { FlugId = flugId };
                free = freePerFlight[flugId];
                free--;
                freePerFlight[flugId] = free;
                ticketId = "T" + nextTicket;
                nextTicket++;
            }

            this.Clients.Caller.BookFlightCompleted(flugId, ticketId, vorname, nachname);
            this.Clients.Group("flug" + flugId).FlightUpdated(flugId, ticketId, free);

            


            if (this.Clients.Caller.Address != null)
            {
                string address = this.Clients.Caller.Address.ToString();
                Debug.WriteLine(address);
            }

            

            return ticketId;
        }



        

        public override System.Threading.Tasks.Task OnDisconnected()
        {
            return Task.Factory.StartNew(() => { 
                this.Clients.All.Status(this.Context.ConnectionId + " disconnected"); 
            });
        }

        public override System.Threading.Tasks.Task OnConnected()
        {
            return Task.Factory.StartNew(() =>
            {
                this.Clients.All.Status(this.Context.ConnectionId + " connected");
            });

        }

        public override System.Threading.Tasks.Task OnReconnected()
        {
            return Task.Factory.StartNew(() =>
            {
                this.Clients.All.Status(this.Context.ConnectionId + " reconnected");
            });

        }
    }
}