﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data.Entity;
using System.Data;
using System.Transactions;

namespace EF_FirstSteps
{



    class BuchungRepository
    {
        public List<HotelBuchung> FindAll()
        {
            using (var ctx = new HotelDbContext())
            {
                return ctx
                        .HotelBuchung
                        .Include(hb => hb.Hotel.Region)
                        .ToList(); 
            }
        }
    }

    class RegionRepository
    {
        public IEnumerable<Region> FindAllRegions()
        {
            using (var ctx = new HotelDbContext())
            {
                var q = ctx.Region;
                return q.ToList();
            }
        }

        public IEnumerable<Region> FindAllRegionsWithNeighbors()
        {
            using (var ctx = new HotelDbContext())
            {
                var q = ctx
                            .Region
                            .Include("Hotel.HotelBuchung");
                            
                return q.ToList();
            }
        }

        public IEnumerable<Region> FindAllRegionsWithNeighbors2()
        {
            using (var ctx = new HotelDbContext())
            {
                var q = ctx
                            .Region
                            .Include(r => r.Hotel.Select(h => h.HotelBuchung));

                return q.ToList();
            }
        }



    }

    class HotelRepository
    {

        public void Delete(Hotel h)
        {
            using (var ctx = new HotelDbContext())
            {
                ctx.Hotel.Attach(h);
                ctx.Hotel.Remove(h);
                ctx.SaveChanges();
                
            }
        }

        public void Insert(Hotel h)
        {
            using (var ctx = new HotelDbContext())
            {
                ctx.Hotel.Add(h);
                ctx.SaveChanges();
            }
        }

        public void Update(Hotel h)
        {
            using (var ctx = new HotelDbContext())
            {
                ctx.Hotel.Attach(h);
                ctx.Entry(h).State = System.Data.EntityState.Modified;
                ctx.SaveChanges();
            }
        }

        public void Update2(Hotel h)
        {
            using (var ctx = new HotelDbContext())
            {
                var hotelInDb = ctx.Hotel.Find(h.HotelId);

                hotelInDb.Bezeichnung = h.Bezeichnung;
                hotelInDb.Sterne = h.Sterne;

                ctx.SaveChanges();
            }
        }



        public void UpdateWithChildrenNegativ(Hotel h)
        {
            using (var ctx = new HotelDbContext())
            {
                ctx.Hotel.Attach(h);
                ctx.Entry(h).State = System.Data.EntityState.Modified;
                foreach (var b in h.HotelBuchung)
                {
                    ctx.Entry(b).State = System.Data.EntityState.Modified;
                }
                ctx.SaveChanges();
            }
        }

        public void Update(Hotel h, IDictionary<object, EntityState> states)
        {
            using (var ctx = new HotelDbContext())
            {
                ctx.Hotel.Attach(h);
                ctx.Entry(h).State = System.Data.EntityState.Modified;
                foreach (var b in h.HotelBuchung)
                {
                    if (states.ContainsKey(b))
                    {
                        ctx.Entry(b).State = states[b];
                    }
                }
                ctx.SaveChanges();
            }
        }


        public void UpdateDemo()
        {
            using (var ctx = new HotelDbContext())
            {
                Hotel h = new Hotel();
                h.Bezeichnung = "Hotel Mama";
                h.RegionId = 3;
                h.Sterne = 4;

                ctx.Hotel.Add(h);
                ctx.SaveChanges();

                h.Sterne++;
                ctx.SaveChanges();

            }
        }

        

        public List<Hotel> FindByMinSterne(int minSterne)
        {
            using (var ctx = new HotelDbContext())
            {
                return ctx
                            .Hotel
                            .Where(h => h.Sterne >= minSterne)
                            .OrderBy(h => h.Sterne)
                            .ToList();
            }
        }


        public List<Hotel> FindByMinSterne2(int minSterne)
        {
            using (var ctx = new HotelDbContext())
            {
                var q = from h in ctx.Hotel
                        where h.Sterne > minSterne
                        orderby h.Sterne
                        select h;

                return q.ToList();
            }
        }

        public Hotel FindById(int hotelId) {
            using (var ctx = new HotelDbContext())
            {
                ctx.Configuration.LazyLoadingEnabled = false;

                return ctx
                            .Hotel
                            .FirstOrDefault(h => h.HotelId == hotelId);
            }
        }

        public Hotel FindByIdWithNighbors(int hotelId)
        {
            using (var ctx = new HotelDbContext())
            {
                return ctx
                            .Hotel
                            .Include("Region")
                            .Include("HotelBuchung")
                            .FirstOrDefault(h => h.HotelId == hotelId);

            }
        }

        public Hotel FindByIdWithNighbors2(int hotelId)
        {
            using (var ctx = new HotelDbContext())
            {
                return ctx
                            .Hotel
                            .Include(h => h.HotelBuchung)
                            .Include(h => h.Region)
                            .FirstOrDefault(h => h.HotelId == hotelId);

            }
        }



        public Hotel FindById2(int hotelId)
        {
            using (var ctx = new HotelDbContext())
            {
                return ctx.Hotel.Find(hotelId);
            }
        }


    }

    class OtherRegionRepository: IDisposable
    {
        private HotelDbContext ctx = new HotelDbContext();

        public IEnumerable<Region> FindAllRegions()
        {
                return ctx.Region;
        }

        public void Dispose()
        {
            ctx.Dispose();            
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            var rep = new HotelRepository();
            var hotel = rep.FindById(10);

            foreach (var b in hotel.HotelBuchung)
            {
                DoSomethingWithBuchung(b);
            }

            var hotelRepository = new HotelRepository();
            var flugRepository = new FlugRepository();

            hotel = new Hotel();
            hotel.Sterne = 1;
            hotel.Bezeichnung = "Test";
            hotel.RegionId = 3;

            using (var ts = new TransactionScope())
            {
                
                hotelRepository.Insert(hotel);
                hotelRepository.Insert(hotel);

                // flugRepository.Book(...);
                Console.WriteLine(IsDistributedTransaction());

                ts.Complete();
            }

            TransactionOptions to = new TransactionOptions();
            to.IsolationLevel = System.Transactions.IsolationLevel.Serializable;
            to.Timeout = TimeSpan.FromMinutes(1);

            using (var ts = new TransactionScope(TransactionScopeOption.Required, to))
            {
                // hotelRepository.Book(...);
                // flugRepository.Book(...);
            }

        }

        private static bool IsDistributedTransaction()
        {
            if (Transaction.Current.TransactionInformation.DistributedIdentifier == Guid.Empty)
            {
                return false;
            }
            return true;
        }

        private static void DoSomethingWithBuchung(HotelBuchung b)
        {
        }
    }
}
