﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;

namespace HotelBuchungen.Data
{
    public enum State {
        Modified, Added, Deleted
    }


    class OtherRegionRepository : IDisposable
    {
        private HotelDbContext ctx = new HotelDbContext();

        public IEnumerable<Region> FindAllRegions()
        {
            return ctx.Region;
        }

        public void Dispose()
        {
            ctx.Dispose();
        }
    }

    public class HotelRepository
    {
        public List<Hotel> FindAll()
        {
            using (var ctx = new HotelDbContext())
            {
                return ctx.Hotel.ToList();
            }
        }

        public Hotel FindById(int hotelId)
        {
            using (var ctx = new HotelDbContext())
            {
                return ctx.Hotel.Include("HotelBuchung").FirstOrDefault(h => h.HotelId == hotelId);
            }
        }

        public void Save(Hotel hotel, IDictionary<HotelBuchung,State> states )
        {
            using (var ctx = new HotelDbContext())
            {
                ctx.Hotel.Attach(hotel);
                ctx.Entry(hotel).State = EntityState.Modified;
                hotel.RegionId = 3;
                hotel.aktiv = "Y";
                foreach (var b in hotel.HotelBuchung.ToList())
                {
                    if (!states.ContainsKey(b)) continue;

                    var state = states[b];
                    ctx.Entry(b).State = ToEntityState(state);
                }

                ctx.SaveChanges();
            }

        }

        private EntityState ToEntityState(State state)
        {
            switch (state)
            {
                case State.Added: return EntityState.Added;
                case State.Modified: return EntityState.Modified;
                case State.Deleted: return EntityState.Deleted;
            }
            throw new Exception("Invalid State");
        }
    }
}