﻿﻿using System;
using System.Data;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.OData;
using System.Web.Http.OData.Routing;
using System.Web.Http.Routing;
using Microsoft.Data.OData;
using Microsoft.Data.OData.Query;
using ODataSample.Models;
using System.Text.RegularExpressions;
using System.Collections.Generic;


namespace ODataSample.Controllers
{
    public class HotelsController : EntitySetController<Hotel, int>
    {
        private HotelDbContext ctx;

        public HotelsController()
        {
            ctx = new HotelDbContext();
        }

        public IQueryable<HotelBuchung> GetHotelBuchungen([FromODataUri] int key)
        {
            return ctx.Hotels.Where(h => h.HotelId == key).SelectMany(h => h.HotelBuchungen);
        }

        protected override Hotel CreateEntity(Hotel entity)
        {
            ctx.Hotels.Add(entity);
            ctx.SaveChanges();


            return entity;
        }

        public override IQueryable<Hotel> Get()
        {
            var r = new List<Hotel>();
            return r.AsQueryable();
        }

        public override void CreateLink(int key, string navigationProperty, [FromBody] Uri link)
        {
            var hotel = ctx.Hotels.Find(key);

            switch (navigationProperty)
            {
                case "HotelBuchungen":

                    var hotelBuchungId = Parse(link.ToString(), @"HotelBuchungen\((\d+)\)");
                    var buchung = ctx.HotelBuchungen.Find(hotelBuchungId);
                    hotel.HotelBuchungen.Add(buchung);
                    ctx.SaveChanges();
                    break;
                case "Region":

                    var regionId = Parse(link.ToString(), @"Regionen\((\d+)\)");

                    var region = ctx.Regionen.Find(regionId);
                    hotel.Region = region;
                    ctx.SaveChanges();
                    break;
                default:
                    throw new ODataException(
                                string.Format("Die Navigations-Eigenschaft Hotel.{0} wird nicht unterstützt.",
                                navigationProperty));
            }

        }

        

        public override void Delete(int key)
        {
            var hotel = ctx.Hotels.Find(key);
            ctx.Entry(hotel).State = System.Data.EntityState.Deleted;
            ctx.SaveChanges();

        }

        public override void DeleteLink(int key, string navigationProperty, [FromBody] Uri link)
        {
            var data = Request.Content.ReadAsStringAsync().Result; 

            switch (navigationProperty)
            {
                case "Region":
                    var hotel = ctx.Hotels.Find(key);
                    hotel.Region = null;
                    ctx.SaveChanges();
                    break;

                default:
                    throw new ODataException(
                                string.Format("Die Navigations-Eigenschaft Hotel.{0} wird nicht unterstützt.",
                                navigationProperty));

            }
            
        }

        public override void DeleteLink(int key, string relatedKey, string navigationProperty)
        {
            switch (navigationProperty)
            {
                case "HotelBuchungen":
                    var id = Convert.ToInt32(relatedKey);
                    var hotel = ctx.Hotels.Find(key);
                    var hb = ctx.HotelBuchungen.Find(id);

                    hotel.HotelBuchungen.Remove(hb);
                    ctx.SaveChanges();

                    break;

                default:
                    throw new ODataException(
                                string.Format("Die Navigations-Eigenschaft Hotel.{0} wird nicht unterstützt.",
                                navigationProperty));

            }
            
        }


        protected override Hotel GetEntityByKey(int key)
        {
            return ctx.Hotels.Find(key);
        }

        protected override int GetKey(Hotel entity)
        {
            return entity.HotelId;
        }



        
        protected override Hotel PatchEntity(int key, Delta<Hotel> patch)
        {
            var hotelInDb = ctx.Hotels.Find(key);
            patch.Patch(hotelInDb);
            ctx.SaveChanges();
            return hotelInDb;

        }

        protected override Hotel UpdateEntity(int key, Hotel update)
        {
            update.HotelId = key;
            ctx.Hotels.Attach(update);
            ctx.Entry(update).State = System.Data.EntityState.Modified;
            ctx.SaveChanges();
            return update;
        }

        public int Parse(string str, string regExp)
        {

            var matches = Regex.Matches(str, regExp);
            if (matches.Count == 0 || matches[0].Groups.Count < 2) throw new ODataException("Kann Zuweisung nicht erstellen!");


            return Convert.ToInt32(matches[0].Groups[1].Value);

        }


        /*

        public void DeleteLink(int key, string relatedKey, string navigationProperty)
        {
            switch (navigationProperty)
            {
                case "HotelBuchungen":
                    var id = Convert.ToInt32(relatedKey);
                    var hotel = ctx.Hotels.Find(key);
                    var hb = ctx.HotelBuchungen.Find(id);

                    hotel.HotelBuchungen.Remove(hb);
                    ctx.SaveChanges();

                    break;

                default:
                    throw new ODataException(
                                string.Format("Die Navigations-Eigenschaft Hotel.{0} wird nicht unterstützt.",
                                navigationProperty));

            }
        }

        public void DeleteLink(int key, string navigationProperty, Uri link)
        {
            switch (navigationProperty)
            {
                case "Region":
                    var hotel = ctx.Hotels.Find(key);
                    hotel.Region = null;
                    ctx.SaveChanges();
                    break;

                default:
                    throw new ODataException(
                                string.Format("Die Navigations-Eigenschaft Hotel.{0} wird nicht unterstützt.",
                                navigationProperty));

            }
        }






        [Queryable]
        public IQueryable<HotelBuchung> GetHotelBuchungen([FromODataUri] int key)
        {
            return ctx.Hotels.Where(h => h.HotelId == key).SelectMany(h => h.HotelBuchungen);
        }

        public Hotel Get(int key)
        {
            return ctx.Hotels.Find(key);
        }


        [Queryable]
        public IQueryable<Hotel> Get()
        {
            return ctx.Hotels.AsQueryable();
        }

        public HttpResponseMessage Post(Hotel h)
        {
            ctx.Hotels.Add(h);
            ctx.SaveChanges();

            var response = Request.CreateResponse(HttpStatusCode.Created, h);

            return response;

        }

        public void Put(int key, Hotel h)
        {
            h.HotelId = key;
            ctx.Hotels.Attach(h);
            ctx.Entry(h).State = System.Data.EntityState.Modified;
            ctx.SaveChanges();
        }

        public void Patch(int key, Delta<Hotel> h)
        {
            var hotelInDb = ctx.Hotels.Find(key);
            h.Patch(hotelInDb);
            ctx.SaveChanges();
        }

        public HttpResponseMessage Delete(int key)
        {
            var hotel = ctx.Hotels.Find(key);
            ctx.Entry(hotel).State = System.Data.EntityState.Deleted;
            ctx.SaveChanges();

            return new HttpResponseMessage(HttpStatusCode.Accepted);
        }


        public int Parse(string str, string regExp)
        {

            var matches = Regex.Matches(str, regExp);
            if (matches.Count == 0 || matches[0].Groups.Count < 2) throw new ODataException("Kann Zuweisung nicht erstellen!");


            return Convert.ToInt32(matches[0].Groups[1].Value);

        }

        [AcceptVerbs("PUT", "POST")]
        public void CreateLink([FromODataUri]int key, string navigationProperty, [FromBody] Uri link)
        {
            var hotel = ctx.Hotels.Find(key);

            switch (navigationProperty)
            {
                case "HotelBuchungen":

                    var hotelBuchungId = Parse(link.ToString(), @"HotelBuchungen\((\d+)\)");
                    var buchung = ctx.HotelBuchungen.Find(hotelBuchungId);
                    hotel.HotelBuchungen.Add(buchung);
                    ctx.SaveChanges();
                    break;
                case "Region":

                    var regionId = Parse(link.ToString(), @"Regionen\((\d+)\)");

                    var region = ctx.Regionen.Find(regionId);
                    hotel.Region = region;
                    ctx.SaveChanges();
                    break;
                default:
                    throw new ODataException(
                                string.Format("Die Navigations-Eigenschaft Hotel.{0} wird nicht unterstützt.",
                                navigationProperty));
            }
        }
*/
    }
}
