﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.OData.Query;
using Microsoft.Data.OData;
using ODataSample.Models;

namespace ODataSample.Controllers
{

    class CustomValidator : System.Web.Http.OData.Query.Validators.FilterQueryValidator
    {

        public override void ValidateBinaryOperatorNode(Microsoft.Data.OData.Query.SemanticAst.BinaryOperatorNode binaryOperatorNode, ODataValidationSettings settings)
        {
            if (binaryOperatorNode.OperatorKind == Microsoft.Data.OData.Query.BinaryOperatorKind.LessThan
                || binaryOperatorNode.OperatorKind == Microsoft.Data.OData.Query.BinaryOperatorKind.LessThanOrEqual)
            {
                throw new ODataException("< oder <= ist nicht erlaubt!");
            }

            base.ValidateBinaryOperatorNode(binaryOperatorNode, settings);
        }

      
    }

    class CustomQueryableAttribute : QueryableAttribute
    {
        public override void ValidateQuery(HttpRequestMessage request, ODataQueryOptions queryOptions)
        {
            if (queryOptions.Filter != null)
            {
                queryOptions.Filter.Validator = new CustomValidator();
            }
            base.ValidateQuery(request, queryOptions);
            
        }
    }

    public class OtherHotelController : ApiController
    {

        private HotelDbContext ctx;

        public OtherHotelController()
        {
            ctx = new HotelDbContext();
        }


        [Queryable(PageSize = 2, AllowedQueryOptions=AllowedQueryOptions.Filter | AllowedQueryOptions.OrderBy | AllowedQueryOptions.Skip)]
        [CustomQueryable(PageSize=2)]
        public IQueryable<Hotel> GetByMinSterne(int minSterne)
        {
            return ctx.Hotels.Where(h => h.Sterne >= minSterne);
        }

        public IQueryable<Hotel> Get(ODataQueryOptions opts)
        {
            if (opts.Filter != null)
            {
                opts.Filter.Validator = new CustomValidator();
            }

            
            
            var settings = new ODataValidationSettings()
            {
                // Initialize settings as needed.
                AllowedFunctions = AllowedFunctions.AllDateTimeFunctions
            };

            opts.Validate(settings);
            if (opts.Filter != null) Debug.WriteLine(opts.Filter.RawValue);

            IQueryable results = opts.ApplyTo(ctx.Hotels.AsQueryable());
            return results as IQueryable<Hotel>;
        }

    }
}
