﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace ExtendingMVC.Metadata
{
    public class CustomMetadataProvider: ModelMetadataProvider
    {

        public override IEnumerable<ModelMetadata> GetMetadataForProperties(object container, Type containerType)
        {
            var result = new List<ModelMetadata>();
            foreach (var p in containerType.GetProperties())
            {
                Func<object> acc = () => p.GetAccessors().First();
                var metadata = new ModelMetadata(this, containerType, acc, p.PropertyType, p.Name);

                BuildMetadata(metadata);
                
                result.Add(metadata);
            }
            return result;
        }

        public override ModelMetadata GetMetadataForProperty(Func<object> modelAccessor, Type containerType, string propertyName)
        {
            var p = containerType.GetProperty(propertyName);
            var metadata = new ModelMetadata(this, containerType, modelAccessor, p.PropertyType, propertyName);
            BuildMetadata(metadata);
            return metadata;
        }

        public override ModelMetadata GetMetadataForType(Func<object> modelAccessor, Type modelType)
        {
            var metadata = new ModelMetadata(this, null, modelAccessor, modelType, null);
            BuildMetadata(metadata);
            return metadata;
        }

        private void BuildMetadata(ModelMetadata metadata)
        {
            // 1. Informationen zu metadata "ermitteln"
            var isRequired = true;
            var displayName = "*" + metadata.PropertyName + "*";

            // 2. Ermittelte Informationen setzen
            metadata.IsRequired = isRequired;
            metadata.DisplayName = displayName;
        }
    }
}