﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using ExtendingMVC.Models;

namespace ExtendingMVC.ModelBinding
{
    public class AddressModelBinder: System.Web.Mvc.IModelBinder
    {
        public object BindModel(System.Web.Mvc.ControllerContext controllerContext, System.Web.Mvc.ModelBindingContext bindingContext)
        {
            var modelName = bindingContext.ModelName;
            var valueProvider = bindingContext.ValueProvider;
            var valueProviderResult = valueProvider.GetValue(modelName);

            var metaData = bindingContext.ModelMetadata;
            var modelState = new ModelState { Value = valueProviderResult };
            bindingContext.ModelState.Add(modelName, modelState);

            var strValue = (string)valueProviderResult.ConvertTo(typeof(string));
            if (strValue == null) return null;
            var strArray = strValue.Split(',');
            var model = new Address();
            
            if (strArray.Length == 3) {
                model.Strasse = strArray[0];
                model.Plz = strArray[1];
                model.Land = strArray[2];
                metaData.Model = model;

            }
            else {
                metaData.Model = valueProviderResult.AttemptedValue;
                modelState.Errors.Add("Ungültiger Wert");
                return model;
            }

            ValidateStrasse(controllerContext, bindingContext, model, modelName);

            return model;

        }

        private void ValidateStrasse(ControllerContext controllerContext, ModelBindingContext bindingContext, Address model, string name)
        {
            var strasseMetadata = bindingContext.PropertyMetadata["Strasse"];
            strasseMetadata.Model = model.Strasse;

            var strasseModelState = new ModelState();
            bindingContext.ModelState.Add(name + ".Strasse", strasseModelState);

            var validators = ModelValidatorProviders.Providers.GetValidators(strasseMetadata, controllerContext);
            foreach (var validator in validators)
            {
                var errors = validator.Validate(model.Strasse);
                foreach (var error in errors)
                {
                    strasseModelState.Errors.Add(error.Message);
                }

            }

        }


    }
}