﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Web;
using System.Web.Http;
using System.Web.Mvc;
using System.Web.Optimization;
using System.Web.Routing;
using ExtendingMVC.Controllers;
using ExtendingMVC.Metadata;
using ExtendingMVC.ModelBinding;
using ExtendingMVC.Models;
using ExtendingMVC.Validation;
using ExtendingMVC.ViewEngine;

namespace ExtendingMVC
{
    // Note: For instructions on enabling IIS6 or IIS7 classic mode, 
    // visit http://go.microsoft.com/?LinkId=9394801


    class CustomDependencyResolver : IDependencyResolver
    {
        private IDictionary<Type, object> singlyRegisteredServices = new Dictionary<Type, object>();
        private IDictionary<Type, List<object>> multiplyRegisteredServices = new Dictionary<Type, List<object>>();
        
        public object GetService(Type serviceType)
        {
            if (singlyRegisteredServices.ContainsKey(serviceType))
            {
                return singlyRegisteredServices[serviceType];
            }
            return null;
        }

        public IEnumerable<object> GetServices(Type serviceType)
        {
            if (multiplyRegisteredServices.ContainsKey(serviceType))
            {
                return multiplyRegisteredServices[serviceType];
            }
            return new List<object>();
        
        }

        public void RegisterService(Type serviceType, object service)
        {
            singlyRegisteredServices[serviceType] = service;
        }

        public void AddService(Type serviceType, object service)
        {
            List<object> services;
            if (multiplyRegisteredServices.ContainsKey(serviceType))
            {
                services = multiplyRegisteredServices[serviceType];
            }
            else
            {
                services = new List<object>();
                multiplyRegisteredServices[serviceType] = services;
            }

            services.Add(service);

        }
    }

    class LoggingDependencyResolver : IDependencyResolver
    {
        public IDependencyResolver Inner { get; private set; }

        public LoggingDependencyResolver(IDependencyResolver inner)
        {
            Inner = inner;
        }

        public object GetService(Type serviceType)
        {
            var result = Inner.GetService(serviceType);
            return result;
        }

        public IEnumerable<object> GetServices(Type serviceType)
        {
            var result = Inner.GetServices(serviceType);
            return result;
        }
    }
    
    public class MvcApplication : System.Web.HttpApplication
    {
        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();


            //ControllerBuilder.Current.SetControllerFactory(new CustomControllerFactory());

            ModelBinders.Binders.DefaultBinder = new SimpleCustomModelBinder();
            ModelBinders.Binders.Add(typeof(int[]), new IntArrayModelBinder());
            ModelBinders.Binders.Add(typeof(Address), new AddressModelBinder());

            ValueProviderFactories.Factories.Add(new DemoXmlValueProviderFactory());
            ValueProviderFactories.Factories.Add(new SessionValueProviderFactory());

            ViewEngines.Engines.Add(new ViewEngineDemo());
            ViewEngines.Engines.Add(new CustomRazorEngine());

            //ModelMetadataProviders.Current = new CustomMetadataProvider();
            ModelMetadataProviders.Current = new CustomDataAnnotationsModelMetadataProvider();

            ModelValidatorProviders.Providers.Add(new CustomModelValidatorProvider());

            //DependencyResolver.SetResolver(new LoggingDependencyResolver(DependencyResolver.Current));
            var dr = new CustomDependencyResolver();
            dr.RegisterService(typeof(HomeController), new HomeController());
            dr.AddService(typeof(IViewEngine), new RazorViewEngine());
            DependencyResolver.SetResolver(dr);

            

            FilterConfig.RegisterGlobalFilters(GlobalFilters.Filters);
            RouteConfig.RegisterRoutes(RouteTable.Routes);
            BundleConfig.RegisterBundles(BundleTable.Bundles);
        }
    }
}