﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace ExtendingMVC.ModelBinding
{
    public class IntArrayModelBinder: System.Web.Mvc.IModelBinder
    {

        public object BindModel(System.Web.Mvc.ControllerContext controllerContext, System.Web.Mvc.ModelBindingContext bindingContext)
        {
            var name = bindingContext.ModelName;
            var valueProvider = bindingContext.ValueProvider;
            var metaData = bindingContext.ModelMetadata;

            var valueProviderResult = valueProvider.GetValue(name);
            var modelState = new ModelState { Value = valueProviderResult };
            bindingContext.ModelState.Add(name, modelState);
            
            var strValue = (string)valueProviderResult.ConvertTo(typeof(string));
            var strArray = strValue.Split(',');
            var intArray = new int[strArray.Length];

            try
            {
                for (int i = 0; i < intArray.Length; i++)
                {
                    intArray[i] = Convert.ToInt32(strArray[i]);
                }
            }
            catch
            {
                
                intArray = null;
                metaData.Model = valueProviderResult.AttemptedValue;
                modelState.Errors.Add("Ungültiger Wert");
                return intArray;
            }

            Validate(controllerContext, metaData, modelState, intArray);

            return intArray;
        }

        private static void Validate(System.Web.Mvc.ControllerContext controllerContext, ModelMetadata metaData, ModelState modelState, int[] intArray)
        {
            var validators = ModelValidatorProviders.Providers.GetValidators(metaData, controllerContext);

            foreach (var validator in validators)
            {
                foreach (var item in intArray)
                {
                    var errors = validator.Validate(item);
                    foreach (var error in errors)
                    {
                        modelState.Errors.Add(error.Message);
                    }
                }
            }
        }
    }
}