﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Xml;
using System.Xml.XPath;

namespace ExtendingMVC.ModelBinding
{

    public class DemoXmlValueProviderFactory : System.Web.Mvc.ValueProviderFactory
    {
        public override IValueProvider GetValueProvider(ControllerContext controllerContext)
        {

            return new DemoXmlValueProvider(controllerContext.HttpContext.Request);
        }
    }

    public class DemoXmlValueProvider: IValueProvider
    {
        private HttpRequestBase request;
        private XPathNavigator nav;
        private HttpSessionStateBase httpSessionStateBase;
        private bool isXML = true;


        public DemoXmlValueProvider(HttpRequestBase httpRequestBase)
        {
            this.request = httpRequestBase;
            try
            {
                var doc = new XPathDocument(request.InputStream);
                nav = doc.CreateNavigator();
            }
            catch
            {
                isXML = false;
            }
        }

       
         
       
        public bool ContainsPrefix(string prefix)
        {
            if (!isXML) return false;

            var xpathQuery = prefix.Replace(".", "/");
            var found = nav.Select(xpathQuery).MoveNext();
            if (found) return found;

            xpathQuery = "*/" + xpathQuery;
            found = nav.Select(xpathQuery).MoveNext();
            return found;
        }

        public ValueProviderResult GetValue(string key)
        {

            if (!isXML) return null;

            var xpathQuery = key.Replace(".", "/");
            var iterator = nav.Select(xpathQuery);
            var found = iterator.MoveNext();

            if (!found)
            {
                xpathQuery = "*/" + xpathQuery;
                iterator = nav.Select(xpathQuery);
                iterator.MoveNext();
            }

            if (!iterator.Current.HasChildren) return null;
            var childrenIterator = iterator.Current.SelectChildren(XPathNodeType.Text);
            if (!childrenIterator.MoveNext()) return null;
            var value = childrenIterator.Current.Value;
            return new ValueProviderResult(value, value, null);
            
            
        }
    }
}