﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace ExtendingMVC.Validation
{
    public class CustomStringLengthValidator: ModelValidator
    {
        public int MinLength { get; set; }
        public int MaxLength { get; set; }

        public CustomStringLengthValidator(ModelMetadata metadata, ControllerContext context, int minLength, int maxLength): base(metadata, context)
        {
            this.MinLength = minLength;
            this.MaxLength = maxLength;
        }

        public override IEnumerable<ModelValidationResult> Validate(object container)
        {
            var result = new List<ModelValidationResult>();
            if (this.Metadata.Model == null) return result;

            var value = this.Metadata.Model.ToString();

            if (value.Length < MinLength || value.Length > MaxLength)
            {
                var propertyName = Metadata.PropertyName;
                var msg = string.Format("Muss zwischen {0} und {1} Zeichen lang sein", MinLength, MaxLength);
                result.Add(new ModelValidationResult { MemberName=propertyName, Message = msg });
            }

            return result;
        }

        public override IEnumerable<ModelClientValidationRule> GetClientValidationRules()
        {
            yield return new ModelClientValidationStringLengthRule("Muss zwischen {0} und {1} Zeichen lang sein", MinLength, MaxLength);
        }
    }
}