﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace ExtendingMVC.Validation
{
    public class PasswordValidationAttribute : ValidationAttribute, IClientValidatable
    {
        public int MinLength { get; set; }
        public bool MustHaveSpecialChar { get; set; }

        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            if (value == null) return ValidationResult.Success;

            var str = value.ToString();
            bool hasDigit = false;
            bool hasLetter = false;
            bool hasSpecialChar = false;

            foreach (char c in str)
            {
                if (char.IsDigit(c)) hasDigit = true;
                else if (char.IsLetter(c)) hasLetter = true;
                else hasSpecialChar = true;
            }

            bool fail = (str.Length < MinLength || !hasDigit || !hasLetter);
            if (!hasSpecialChar && MustHaveSpecialChar)
            {
                fail = true;
            }

            if (fail)
            {
                return new ValidationResult(base.ErrorMessageString + "!");
            }

            return ValidationResult.Success;
        }

        public IEnumerable<ModelClientValidationRule> GetClientValidationRules(ModelMetadata metadata, ControllerContext context)
        {
            var rule = new ModelClientValidationRule();
            rule.ErrorMessage = base.ErrorMessageString + "!";
            rule.ValidationParameters["minlength"] = MinLength;
            rule.ValidationParameters["musthavespecialchar"] = MustHaveSpecialChar;
            rule.ValidationType = "password";

            var result = new List<ModelClientValidationRule>();
            result.Add(rule);
            return result;
        }
    }
}