﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net.Http;
using System.Web;
using System.Web.Http.Filters;

namespace ExtWebApi.Filter
{

    public class SampleExceptionFilterAttribute : ExceptionFilterAttribute
    {
        public override void OnException(HttpActionExecutedContext actionExecutedContext)
        {
            Debug.WriteLine("SampleExceptionFilterAttribute.OnException");
            Debug.WriteLine("Exception: " + actionExecutedContext.Exception.Message);
        }
    }

    public class SampleAuthorizationFilterAttribute : AuthorizationFilterAttribute
    {
        public override void OnAuthorization(System.Web.Http.Controllers.HttpActionContext actionContext)
        {
            Debug.WriteLine("SampleAuthorizationFilterAttribute.OnAuthorization");
            Debug.WriteLine("Authorization-Header" + actionContext.Request.Headers.Authorization);
        }
    }

    public class SampleActionFilterAttribute : ActionFilterAttribute
    {

        public override void OnActionExecuting(System.Web.Http.Controllers.HttpActionContext actionContext)
        {
            var request = actionContext.Request;
            var response = actionContext.Response;
            var action = actionContext.ActionDescriptor;
            var actionArguments = actionContext.ActionArguments;
            var modelState = actionContext.ModelState;

            Debug.WriteLine("SampleActionFilterAttribute.OnActionExecuting");
            Debug.WriteLine("HTTP Method: " + request.Method);
            Debug.WriteLine("Uri: " + request.RequestUri);
            Debug.WriteLine("ActionName: " + action.ActionName);
            Debug.WriteLine("ReturnType: " + action.ReturnType);
            Debug.WriteLine("Arguments: " + actionArguments.Count);
            Debug.WriteLine("IsValid: " + modelState.IsValid);

        }
        
        public override void OnActionExecuted(HttpActionExecutedContext actionExecutedContext)
        {
            var request = actionExecutedContext.Request;
            var response = actionExecutedContext.Response;
            var actionContext = actionExecutedContext.ActionContext;
            var exception = actionExecutedContext.Exception;

            Debug.WriteLine("SampleActionFilterAttribute.OnActionExecuting");
            Debug.WriteLine("HTTP Method: " + request.Method);
            Debug.WriteLine("Uri: " + request.RequestUri);
            Debug.WriteLine("ActionName: " + actionContext.ActionDescriptor.ActionName);

            if (exception != null)
            {
                Debug.WriteLine("Exception: " + exception.Message);
            }

            if (response != null && response.Content != null)
            {
                Debug.WriteLine("Content: " + response.Content.ReadAsStringAsync().Result);
            }

        }

    }
}