﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Http.Controllers;
using System.Web.Http.Metadata;
using System.Web.Http.ModelBinding;
using System.Web.Http.Validation;
using ExtWebApi.Models;


namespace ExtWebApi.ModelBinder
{
    public class AdresseBinder : IModelBinder
    {
        public bool BindModel(System.Web.Http.Controllers.HttpActionContext actionContext, ModelBindingContext bindingContext)
        {
            
            var name = bindingContext.ModelName;
            var valueProvider = bindingContext.ValueProvider;
            var metaData = bindingContext.ModelMetadata;

            var valueProviderResult = valueProvider.GetValue(name);
            var modelState = new ModelState { Value = valueProviderResult };
            bindingContext.ModelState.Add(name, modelState);

            var strValue = (string)valueProviderResult.ConvertTo(typeof(string));
            var strArray = strValue.Split(',');
            var adresse = new Adresse();

            try
            {
                adresse.Strasse = strArray[0];
                adresse.Plz = strArray[1];

                bindingContext.Model = adresse;

                var strasseMetaData = metaData.Properties.Where(m => m.PropertyName == "Strasse").FirstOrDefault();
                var plzMetaData = metaData.Properties.Where(m => m.PropertyName == "Plz").FirstOrDefault();

                Validate(actionContext, strasseMetaData, modelState, adresse.Strasse);
                Validate(actionContext, plzMetaData, modelState, adresse.Plz);

                return true;

            }
            catch
            {
                
                metaData.Model = valueProviderResult.AttemptedValue;
                modelState.Errors.Add("Ungültiger Wert");

                return false;
            }

            

            
        }

        private static void Validate(HttpActionContext actionContext, ModelMetadata metaData, ModelState modelState, string element)
        {

            

            var modelValidatorProvidersObj = actionContext.ControllerContext.Configuration.Services.GetServices(typeof(ModelValidatorProvider));

            var modelValidatorProviders = new List<ModelValidatorProvider>();
            modelValidatorProvidersObj.ToList().ForEach(v => modelValidatorProviders.Add((ModelValidatorProvider) v));


            var validators = metaData.GetValidators(modelValidatorProviders);

            foreach (var validator in validators)
            {
                var errors = validator.Validate(metaData, element);
                foreach (var error in errors)
                {
                    modelState.Errors.Add(error.Message);
                }
            
            }
        }
    }
}