﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Http.Validation;

namespace ExtWebApi.Validation
{
    public class AnschriftValidator : ModelValidator
    {
        public AnschriftValidator(IEnumerable<ModelValidatorProvider> validatorProviders)
            : base(validatorProviders)
        {
        }

        public override IEnumerable<ModelValidationResult> Validate(System.Web.Http.Metadata.ModelMetadata metadata, object container)
        {
            var result = new List<ModelValidationResult>();
            var anschrift = metadata.Model.ToString();
            var parts = anschrift.Split(' ');
            
            if (parts.Length >= 2 && !string.IsNullOrEmpty(parts[0]) && !string.IsNullOrEmpty(parts[1]))
            {
                CheckMinLengthWithoutNumber(metadata, result, parts);
                return result;
            }

            var error = new ModelValidationResult();
            error.MemberName = metadata.PropertyName;
            error.Message = "Keine gültige Adresse im Format 'strassenname hausnummer'";
            result.Add(error);
            return result;

        }

        private static void CheckMinLengthWithoutNumber(System.Web.Http.Metadata.ModelMetadata metadata, List<ModelValidationResult> result, string[] parts)
        {
            if (metadata.AdditionalValues.ContainsKey("minLengthWithoutNumber"))
            {
                var minLengthWithoutNumber = Convert.ToInt32(metadata.AdditionalValues["minLengthWithoutNumber"]);
                if (parts[0].Length < minLengthWithoutNumber)
                {
                    var error = new ModelValidationResult();
                    error.MemberName = metadata.PropertyName;
                    error.Message = "Der Name der Strasse muss aus min. " + minLengthWithoutNumber + " Zeichen bestehen.";
                    result.Add(error);
                }
            }
        }
    }
}